/*

 * icotoxpm - convert MS-Windows icon file to XPM file
 *
 * version 1.2
 *
 * Copyright (c) 1996 KiDong Lee
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/stat.h>

#define TRUE 1
#define FALSE 0
#define TRANSPARENT 16
#define ICONFILESIZE 766
#define APPNAME "icotoxpm"
#define START 126
#define ERROR -1
#define OK 1

void
usage (void)
{
  fprintf (stderr, "usage : %s [filename]\n", APPNAME);
  exit (1);
}

long
GetFileSize (char *filename)
{
  struct stat buf;

  if (stat (filename, &buf) != 0)
    return -1;

  return buf.st_size;
}

int
MatchSuffix (char *s1, char *s2)
{
  if (strlen (s1) < strlen (s2))
    return FALSE;

  if (strcmp (&s1[strlen (s1) - strlen (s2)], s2) == 0)
    return TRUE;
  else
    return FALSE;
}

int
FileExist (char *filename)
{
  FILE *fp;

  if ((fp = fopen (filename, "r")) == NULL)
    return ERROR;

  fclose (fp);

  return OK;
}

int
main (int argc, char **argv)
{
  const char color_sym[17] = "0rgybmcaARGYBMCW ";
  char filename[BUFSIZ];
  int array[32][32];
  int n, i, x, y, color, mask;
  FILE *fpin, *fpout;

  if (argc < 2)
    {
      usage ();
      exit (1);
    }

  /* multiple argument */
  for (n = 1; n < argc; n++)
    {
      if (FileExist (argv[n]) == ERROR)
	{
	  fprintf (stderr, "%s: %s: %s.\n",
		   APPNAME, argv[n], strerror (errno));
	  exit (2);
	}

      /* identify icon file */
      if (MatchSuffix (argv[n], ".ico") == FALSE)
	{
	  fprintf (stderr, "%s: %s is not a icon file.\n",
		   argv[0], argv[n]);
	  continue;
	}

      if (GetFileSize (argv[n]) != ICONFILESIZE)
	{
	  fprintf (stderr, "%s: %s is not a Windows 3.1 icon file.\n"
		   ,argv[0], argv[n]);
	  continue;
	}

      /* open icon(*.ico) file */
      fpin = fopen (argv[n], "r");
      if (fpin == NULL)
	{
	  fprintf (stderr, "%s: %s: %s.\n",
		   argv[0], argv[1], strerror (errno));
	  exit (3);
	}

      /* generate xpm filename */

      strcpy (filename, argv[n]);

      filename[strlen (filename) - 4] = '\0';
      strcat (filename, ".xpm");

      /* open xpm file */
      fpout = fopen (filename, "w");
      if (fpout == NULL)
	{
	  fprintf (stderr, "%s: %s.\n", APPNAME, strerror (errno));
	  exit (1);
	}
      printf ("Converting to %s ...\n", filename);

      /* write xpm header */
      filename[strlen (argv[n]) - 4] = '\0';
      fprintf (fpout, "/* XPM */\n\
static char *%s[] = {\n\
/* %s pixmap\n\
* width height ncolors chars_per_pixel */\n\
\"32 32 17 1 \",\n\
/* colors */\n\
\"%c c black       m black         s s_black \",\n\
\"%c c red4        m black         s s_darkred \",\n\
\"%c c green4      m black         s s_darkgreen \",\n\
\"%c c yellow4     m black         s s_darkyellow \",\n\
\"%c c blue4       m black         s s_darkblue \",\n\
\"%c c magenta4    m black         s s_darkmagenta \",\n\
\"%c c cyan4       m black         s s_darkcyan \",\n\
\"%c c gray50      m black         s s_darkgray \",\n\
\"%c c gray75      m white         s s_gray \",\n\
\"%c c red         m white         s s_red \",\n\
\"%c c green       m white         s s_green \",\n\
\"%c c yellow      m white         s s_yellow \",\n\
\"%c c blue        m white         s s_blue \",\n\
\"%c c magenta     m white         s s_magenta \",\n\
\"%c c cyan        m white         s s_cyan \",\n\
\"%c c white       m white         s s_white \",\n\
\"%c c None        m white         s None \",\n\
/* pixels */\n", filename, filename, color_sym[0], color_sym[1], color_sym[2],
	       color_sym[3], color_sym[4], color_sym[5], color_sym[6],
	       color_sym[7], color_sym[8], color_sym[9], color_sym[10],
	       color_sym[11], color_sym[12], color_sym[13],
	       color_sym[14], color_sym[15], color_sym[16]);

      /* read icon file (XOR mask) */
      for (y = 0; y < 32; y++)
	{
	  fseek (fpin, START + 16 * (31 - y), SEEK_SET);

	  for (x = 0; x < 32; x += 2)
	    {
	      color = fgetc (fpin);

	      array[x][y] = color >> 4;
	      array[x + 1][y] = color & 0x0f;
	    }
	}

      /* transparent area (AND mask) */
      for (y = 0; y < 32; y++)
	{
	  fseek (fpin, START + (32 * 32 / 2) + 4 * (31 - y),
		 SEEK_SET);

	  for (x = 0; x < 32; x += 8)
	    {
	      mask = fgetc (fpin);

	      for (i = 0; i < 8; i++)
		if (((mask >> (7 - i)) & 1) == 1)
		  array[x + i][y] = TRANSPARENT;
	    }
	}

      /* write xpm file */
      for (y = 0; y < 32; y++)
	{
	  fprintf (fpout, "\"");
	  for (x = 0; x < 32; x += 2)
	    {
	      color = fgetc (fpin);

	      fputc (color_sym[array[x][y]], fpout);
	      fputc (color_sym[array[x + 1][y]], fpout);
	    }
	  fprintf (fpout, "\"");
	  if (y != 1)
	    fprintf (fpout, ",\n");
	  else
	    fprintf (fpout, "\n");
	}
      fprintf (fpout, "};");

      /* close files */
      fclose (fpout);
      fclose (fpin);
    }
  exit (0);
}
